;;   CHTXT.LSP
;;   Copyright (C) 1994  by Electronic Data Systems, Inc.
;;                          750 Tower Drive. 
;;                          Troy, MI 48084
;;
;;   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
;;   ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE ARE HEREBY
;;   DISCLAIMED.
;;
;;   Larry Adams
;;   12/14/94
;;
;;--------------------------------------------------------------------------;
;;   This utility builds a list of all text in a model the height, block names,
;;   text styles, and number of text entities, and then allows you to change
;;   text height, or style assigned to the text.  This works both inside
;;   and outside of blocks.
;; 
(princ "\nCHTXT.LSP version 1.0. (C) 1994 EDS Inc.")
(princ "\nStart command by typing in CHTXT.")

(if (null def*err*)                       ;saves system default error function
	(setq def*err* *error*)
)
(defun sys*err* ()                        ;function restores system error.
	(setq *error* def*err*)
)
(defun new*err* ()                        ;function activates generic
	(defun *error* (m)                     ;error handling routine.
		(if (/= m "Function cancelled") (princ (strcat "\nError: " m)))
		(if fp (setq fp (close fp)))
		(sys*err*)
		(prin1)
	) ; end *error*
) ; end new*err*

;; Function TRIM
;; 
(defun trim( input_string / count )
	(setq count (strlen input_string))
	(while (/= count 0)
		(if (= (substr input_string count 1) " ")
			(setq count (- count 1))
			(progn
				(setq input_string (substr input_string 1 count))
				(setq count 0)
			) ; end progn
		) ; endif
	) ; end while
	input_string
) ; end trim

;; Function: DXF
;; Returns the DXF value of group "x"
;;
(defun dxf (x table)
	(cdr (assoc x table))
)

(defun gettext ( / ename tbl txtinfo liststr blank)
	(princ "\nSearching BLOCKS for Text")
	(setq g_txtlist nil)
	(setq tbl (tblnext "BLOCK" T))
	(while tbl
		(setq ename (dxf -2 tbl))
		(srchmdl ename (dxf 2 tbl) "BLOCKS")
		(setq tbl (tblnext "BLOCK"))
	) ; endwhile

	(princ "\nSearching ENTITIES for Text")
	(setq ename nil) 
	(srchmdl ename "-" "ENTITIES")	

	(if (= (type g_txtlist) 'LIST)
		(progn
			;; Create internal processing display list
			(setq blank "                               ")
			(setq g_intlist nil)
			(foreach txtinfo g_txtlist
				(setq blkins (cadr txtinfo))
				(setq blkhght (trim (nth 0 (car txtinfo))))
				(if (> (strlen blkhght) 8)
					(setq blkhght (substr blkhght 1 9))
					(setq blkhght (strcat (substr blank 1 (- 9 (strlen blkhght))) blkhght))
				)
				(setq blkname (nth 1 (car txtinfo)))
				(setq blkstyle (nth 2 (car txtinfo)))
				(setq liststr 
					(strcat
						blkhght  (chr 9)
						(substr (strcat blkname blank) 1 31) (chr 9)
						(substr (strcat blkstyle blank) 1 31) (chr 9)
						(substr (strcat (itoa blkins) blank) 1 6)
					)
				)
				(setq g_intlist (cons liststr g_intlist)) 
			) ; endforeach

			;; Sort internal text list by height, block
			;; and style name and then build the display
			;; list.
			(if (= (type g_intlist) 'LIST)
				(setq g_intlist (acad_strlsort g_intlist))
			) ; endif
			(setq g_disptlist nil)
			(foreach txtinfo g_intlist
				(setq g_disptlist 
					(cons (strcat
						(substr txtinfo 1 9) (chr 9)
						(substr txtinfo 11 20) (chr 9)
						(substr txtinfo 43 20) (chr 9)
						(substr txtinfo 75 6))
						g_disptlist
					)
				)
			) ; endforeach
			(if (= (type g_disptlist) 'LIST)
				(setq g_disptlist (reverse g_disptlist))
			) ; endif
		) ; endprogn
	) ; endif
) ; end gettext

;; Function: CMPNBLD
;; Compare the block name, style and height
;; from input list and add to the global display
;; list.  Also, counts the number of times the 
;; text is inserted in drawing
;;
(defun compnbld ( inlist / complist )
	(if (null (setq complist (assoc inlist g_txtlist)))
		(setq g_txtlist (cons (list inlist 1) g_txtlist))
		(setq g_txtlist (subst (list (car complist) (+ 1 (cadr complist))) complist g_txtlist))
	) ; endif
) ; end compnbld

;; Function: SRCHMDL
;; Search model for text entities in the specified
;; table (ENTITIES, BLOCKS), call COMPNBLD to
;; add the text entities to the global text display list.
;;
(defun srchmdl ( ename blkname tabltype / ent cntr ss sscntr)
	(setq cntr 0)
	(setq sscntr 0)
	(if (= tabltype "ENTITIES")
		(progn
			(setq ss (ssget "X" '((0 . "TEXT"))))
			(if ss
				(setq ename (ssname ss sscntr))
				(setq ename nil)
			) ; endif
		) ; endprogn
	) ; endif
	(while ename
		(if (= cntr 15)
			(progn
				(princ ".")
				(setq cntr 0)
			) ; endprogn
			(setq cntr (+ 1 cntr))
		) ; endif
		(setq ent (entget ename))
		(if (= (dxf 0 ent) "TEXT")
			(compnbld (list (rtos (dxf 40 ent)) blkname (dxf 7 ent)))
		) ; endif
		(if (= tabltype "ENTITIES")
			(progn
				(setq sscntr (+ 1 sscntr))
				(setq ename (ssname ss sscntr))
			) ; endprogn
			(setq ename (entnext ename))
		) ; endif
	) ; endwhile
) ; end srchmdl

;; Function GETPARMS
;;
;; This function is called in response to an "OK"
;; click on the main dialog panel.  Its job is to get
;; the input variables and determine if processing
;; can continue, else messages will be displayed
;; on the error tile "MSG".
;;
(defun getparms( / dodone retcode contin)
	(setq dodone F)
	(setq contin F)
	(setq g_how (get_tile "how"))
	(setq g_what (get_tile "what"))
	(setq g_txtsel (get_tile "txtlist"))
	(setq g_nheight (atof (get_tile "nheight")))
	(setq g_stylsel (get_tile "txtstyl"))
	(setq g_takehght (get_tile "takehght"))
	(set_tile "msg" "")
	(if (= (strlen g_txtsel) 0)
		(set_tile "msg" (setq msg "ERROR: No entries are selected, Select entries and reprocess."))
		(if (= g_how "chgsz")
			(if (<= g_nheight 0.0)
				(set_tile "msg" (setq msg "ERROR: Resizing or rescaling of text requires value greater than zero."))
				(setq dodone T)
			) ; endif
			(if (= g_stylsel nil)
				(set_tile "msg" (setq msg "ERROR: This option requires you to select a text style."))
				(setq dodone T)
			) ; endif
		) ; endif
	) ; endif
	(if dodone
		(progn
			(if (not (new_dialog "rusure" dcl_id)) (exit))
			(set_tile "return" "1")
			(set_tile "regen" "1")
			(action_tile "accept" "(getcontvals)(done_dialog 1)")
			(action_tile "cancel" "(done_dialog 0)")
			(setq retcode (start_dialog))
			(if (= retcode 1)
				(progn
					(setq reprocess T)
					(setq contin T)
				) ; endprogn
				(setq contin F)
			) ; endif
		) ; endprogn
	) ; endif
	contin
) ; end getparms

;; Function: GetFromCSV
;;
;; This generic function will return the field specified from
;; a CSV or other delimited file.  Input variables are as
;; follows:
;;		tpos			-	The field position in the record to be returned
;;		tstring		-	The record string to extract the variable from
;;		separater	-	The field separator for the CSV record.
;;
(defun GetFromCSV ( tpos tstring separater / scnt ccnt error slen done tstr tchr )
	(setq error F)
	(setq ccnt 1)
	(setq scnt 1)
	(setq done F)
	(setq tstr "")
	(setq slen (strlen tstring))
	(while (and (<= scnt tpos)
		(= error F)
		(< ccnt slen)
		(= done F))
		(setq tchr (substr tstring ccnt 1))
		(if (= tchr separater)
			(if (= scnt tpos)
				(setq done T)
				(progn
					(setq tstr "")
					(setq scnt (+ 1 scnt))
				) ; endprogn
			) ; endif
			(if (= tpos scnt)(setq tstr (strcat tstr tchr)))
		) ; endif
		(if (= ccnt slen)
			(if (= scnt tpos)
				(setq done T)
				(setq error T)
			) ; endif
			(setq ccnt (+ 1 ccnt))
		) ; endif
	) ; endwhile		
	tstr
) ; end GetFromCSV

;; Function GETCONTVALS
;;
;; This function is called in response to the "OK"
;; click on the RUSURE dialog.  It sets global
;; variables for further processing.
;;
(defun getcontvals ( / )
	(setq g_return (get_tile "return"))
	(setq g_regen (get_tile "regen"))
) ; end getcontvals

;; Function: PROCESS_IT
;;
;; This function takes all of the selected text
;; entities and changes their text values to the
;; new ones.
;;
(defun process_it ( / ent ename sel# readl readl# tbl ss sscnt ssnum 
	expr data blockname stylename height )
	(if (= g_how "chgsz")
		(if (= g_what "act")
			(setq g_op g_nheight)
			(setq g_op (list '* g_nheight '(cdr (assoc 40 ent))))
		) ; endif
		(progn
			(setq g_style (GetFromCSV 1 (nth (atoi g_stylsel) g_txtstyl) ","))
			(if (= g_takehght "1")
				(setq g_op (atof (GetFromCSV 2 (nth (atoi g_stylsel) g_txtstyl) ",")))
				(setq g_op "")
			) ; endif
		) ; endprogn
	) ; endif
	(setq readl g_txtsel)
	(setq sel# (read g_txtsel))
	(while (/= (strlen readl) 0)
		(setq data (nth sel# g_intlist))
		(setq blockname (trim (GetFromCSV 2 data (chr 9))))
		(setq stylename (trim (GetFromCSV 3 data (chr 9))))
		(setq height (trim (GetFromCSV 1 data (chr 9))))
		(if (= blockname "-")
			(progn
				(setq expr 
					(list 
						'(0 . "TEXT") 
						(cons 7 stylename) 
						'(-4 . "<=") (cons 40 (+ (atof height) 0.00005)) 
						'(-4 . ">=") (cons 40 (- (atof height) 0.00005))
					)
				)
				(setq ss (ssget "X" expr))
				(setq sscnt 0)
				(setq ssnum (sslength ss))
				(while (<= (+ 1 sscnt) ssnum)
					(setq ename (ssname ss sscnt))
					(setq ent (entget ename))
					(if (= g_how "chgsz")
						(setq ent (subst (cons 40 (eval g_op)) (assoc 40 ent) ent))
						(progn
							(setq ent (subst (cons 7 g_style) (assoc 7 ent) ent))
							(if (= g_takehght "1")
								(setq ent (subst (cons 40 (eval g_op)) (assoc 40 ent) ent))
							) ; endif
						) ; endprogn
					) ; endif
					(entmod ent)
					(setq sscnt (+ 1 sscnt))
				) ; endwhile
			) ; endprogn
			(progn
				(setq tbl (tblsearch "BLOCK" blockname))
				(if tbl
					(progn
						(setq ename (dxf -2 tbl))
						(while ename
							(setq ent (entget ename))
							(if (= (dxf 0 ent) "TEXT")
								(progn
									(if (= g_how "chgsz")
										(setq ent (subst (cons 40 (eval g_op)) (assoc 40 ent) ent))
										(progn
											(setq ent (subst (cons 7 g_style) (assoc 7 ent) ent))
											(if (= g_takehght "1")
												(setq ent (subst (cons 40 (eval g_op)) (assoc 40 ent) ent))
											) ; endif
										) ; endprogn
									) ; endif
									(entmod ent)
								) ; endprogn
							) ; endif
							(setq ename (entnext ename))
						) ; endwhile
					) ; endprogn
				) ; endif
			) ; endprogn
		) ; endif
		(princ (strcat "\n" height " : " blockname " : " stylename " <<- PROCCESSED"))
		(setq readl# 1)
		(while (and (/= " " (substr readl readl# 1))
		(/= "" (substr readl readl# 1)))
			(setq readl# (+ readl# 1))
		) ; end while
		(setq readl (substr readl (+ readl# 1)))
		(setq sel# (read readl))
	) ; endwhile
) ; end process_it

;; Function CHGMETH
;;
(defun chgmeth( / )
	(if (= (get_tile "chgsz") "1")
		(progn
			(mode_tile "txtprop" 0)
			(mode_tile "stylprop" 1)
		) ; endprogn
		(progn
			(mode_tile "txtprop" 1)
			(mode_tile "stylprop" 0)
		) ; endprogn
	) ; endif
) ; end chgmeth
	
;; Main Procedure CHTXT
;;
(defun C:CHTXT (/ luit cmdekho msg bdlst contin
	dat dcl_id blkins blkhght blkname blkstyle g_txtstyl reprocess
	g_txtsel g_txtlist g_how g_what g_nheight g_stylsel g_takehght
	g_return g_regen g_op g_style g_intlist g_disptlist )

	;; turn off command echoing
	;; set precision down to non roundoff levels
	;; set regeneration to optimal levels
	(setq cmdekho (getvar "cmdecho"))
	(setq luit (getvar "luprec"))
	(setvar "cmdecho" 0)
	(setvar "luprec" 4)
	(command "regenauto" "OFF")
	
	;; Set required variables
	;;
	(new*err*)
	(setq msg "")
	(setq g_nheight 1.0)
	(setq g_how "chgsz")
	(setq g_what "act")
	(setq g_txtsel "")
	(setq g_stylsel "")
	(setq g_takehght 0)
	(setq reprocess T)
	(setq bdlst nil)
	(setq g_txtstyl nil)
	(setq contin 1)

	;; Find all text styles in DWG file
	;;
	(setq 	dat (tblnext "STYLE" T))
	(while dat
		(setq g_txtstyl 
			(cons 
				(strcat (cdr (assoc 2 dat)) ", " 
					(rtos (dxf 40 dat)) ", " 
					(rtos (dxf 50 dat)) ", " 
					(strcase (dxf 3 dat) F)
				)
				g_txtstyl
			)
		)
		(setq dat (tblnext "STYLE"))
	) ; endwhile

	;; Enter main processing loop.  Continue loop until user
	;; cancels or quits
	;;
	(if (null (setq dcl_id (load_dialog "chtxt.dcl")))            ;load dcl file
		(setq contin nil)
	) ; endif

	(while contin
		(if reprocess (gettext))    ; get all text entities from model
		;; Present Dialog (MAINLINE)
		;;
		(if (not (new_dialog "seltxt" dcl_id)) (exit))

		;; create text display list
		(if (list g_disptlist)
			(progn
				(start_list "txtlist" 3)
				(mapcar 'add_list g_disptlist)
				(end_list)
			) ; endprogn
		) ; endif

		;; create style display list
		(if (list g_txtstyl)
			(progn
				(start_list "txtstyl" 3)
				(mapcar 'add_list g_txtstyl)
				(end_list)
			) ; endprogn
		) ; endif

		;; preprocessing and callback assignments
		(set_tile "nheight" (rtos g_nheight))
		(set_tile "how" g_how)
		(set_tile "what" g_what)
		(if reprocess
			(set_tile "txtlist" (setq g_txtsel ""))
			(set_tile "txtlist" g_txtsel)
		) ; endif
		(chgmeth)

		(action_tile "how" "(chgmeth)")
		(action_tile "accept" "(if (= (getparms) T)(done_dialog 1))")
		(action_tile "cancel" "(done_dialog 0)")

		;; activate dialog
		(setq retcode (start_dialog))

		;; process callback results
		(if (= retcode 1)
			(progn
				(process_it)
				(setq reprocess T)
				(if (= g_regen "1") (command "regen"))
				(if (= g_return "1") 
					(setq contin T)
					(setq contin F)
				) ; endif
			) ; endprogn
			(progn
				(sys*err*)
				(setq contin F)
				(prompt "\nCHBTX Cancelled.")
				(prin1)
			) ; end progn
		) ; endif
	) ; endwhile

	(princ "\nThanks for Using CHTXT!")
	(command "regenauto" "on")
	(setvar "cmdecho" cmdekho)
	(setvar "luprec" luit)
	(prin1)
) ; end c:chtxt
(prin1)
